/* This module controls the derandomizer buffer and the 
storing of derandomizer addresses into the pointerFIFO */   
// The logic that handles Rejected L1Aa and Empty-event generation
// was simulated ok in version 08 and reported here.

`include "define.v"
module DaqCntr
 ( input  clk,   rst, L1A,  CalibTrig,  RdyAck,
   input  EmEv_from_pf,  pendingL1A_b,  EventBufferEmpty,
   input  [4:0] NS, NPS, // Number of Samples and Pre-Samples per L1A
   input  [`COMPR_LUT_DATA_WIDTH-1:0] DaqMuxOut,
   input  [17:0] 
          Threshold1,  Threshold2,  Threshold3,  Threshold4, 
          Threshold5,  Threshold6,  Threshold7,  Threshold8, 
          Threshold9,  Threshold10, Threshold11, Threshold12, 
          Threshold13, Threshold14, Threshold15, Threshold16, 
          Threshold17, Threshold18, Threshold19, Threshold20, 
          Threshold21, Threshold22, Threshold23, Threshold24, 
   input  [`DRND_ADDR_WIDTH-1:0]  DrndPointer, // from pointer_fifo 
   output [`DRND_ADDR_WIDTH-1:0]  DrndRdAddr,      
   output reg DrndWe, EmEv_to_fifo, RL_to_fifo, legal_L1A,
   output reg EventBufferPush, pendingL1A_20d, BZ, OW, EvtRdy4Tx,
   output reg [7:0] TPwordCount,
   output reg [8:0] DAQwordCount,
   output reg [`DRND_ADDR_WIDTH-1:0] DrndWrAddr, 
   output reg [`DAQ_MUX_SEL-1:0]   ChanSel      );

 parameter                                  // machine states
 IDLE        = 21'b 000000000000000000001,  //         
 N_CASE      = 21'b 000000000000000000010,  // Normal Case
 N_EVAL_TP   = 21'b 000000000000000000100,  // Evaluate TP for ZeroSup of TP
 N_ONE_TP    = 21'b 000000000000000001000,  // Acquire one TP word   
 N_NS_TP     = 21'b 000000000000000010000,  // Acquire NS TP words
 N_NEXT_TP   = 21'b 000000000000000100000,  // Go to next TP-channel
 N_START_DAQ = 21'b 000000000000001000000,  //
 N_NOP       = 21'b 000000000000010000000,  // 
 N_EVAL_DAQ  = 21'b 000000000000100000000,  // Evaluate TP for ZeroSup of DAQ
 N_NS_DAQ    = 21'b 000000000001000000000,  // Acquire NS DAQ words
 N_NEXT_DAQ  = 21'b 000000000010000000000,  // Go to next DAQ-channel
 D_CASE      = 21'b 000000000100000000000,  // Debug Case
 D_EVAL_TP   = 21'b 000000001000000000000,  //     
 D_ONE_TP    = 21'b 000000010000000000000,  //       
 D_NEXT_TP   = 21'b 000000100000000000000,  //    
 D_START_DAQ = 21'b 000001000000000000000,  //
 D_NOP       = 21'b 000010000000000000000,  // 
 D_EVAL_DAQ  = 21'b 000100000000000000000,  //     
 D_NS_DAQ    = 21'b 001000000000000000000,  //     
 D_NEXT_DAQ  = 21'b 010000000000000000000,  //
 WAIT_ACK    = 21'b 100000000000000000000;  // Wait ackowledgment from DaqFormatter   
   reg  [20:0] LoadingState;                // State variable. [RANGE] is important

   reg  L1A_d1, L1A_d2, internal_legal_L1A, RL; // RL=Rejected L1A
   reg  EvBufPushPre;
   reg  [4:0]  L1As_in_History;
   reg  [4:0]  SampleCntr;
   reg  [5:0]  stretch_cntr;
   reg  [19:0] pendingL1Apipe;
   reg  [23:0] L1A_history;
   reg  [`DRND_ADDR_WIDTH-1:0]  WordCount_d, LatchedDrndPointer;
   wire [`DRND_ADDR_WIDTH-1:0]  WordCount;
   reg  [8:0] DTh, TTh;
   wire [8:0] DaqTh[24:1];  // 2-dimensional array of wires
   wire [8:0] TpTh[24:1];
   assign
         DaqTh[1]  = Threshold1[8:0],   TpTh[1]  = Threshold1[17:9], 
         DaqTh[2]  = Threshold2[8:0],   TpTh[2]  = Threshold2[17:9], 
         DaqTh[3]  = Threshold3[8:0],   TpTh[3]  = Threshold3[17:9], 
         DaqTh[4]  = Threshold4[8:0],   TpTh[4]  = Threshold4[17:9], 
         DaqTh[5]  = Threshold5[8:0],   TpTh[5]  = Threshold5[17:9], 
         DaqTh[6]  = Threshold6[8:0],   TpTh[6]  = Threshold6[17:9], 
         DaqTh[7]  = Threshold7[8:0],   TpTh[7]  = Threshold7[17:9], 
         DaqTh[8]  = Threshold8[8:0],   TpTh[8]  = Threshold8[17:9], 
         DaqTh[9]  = Threshold9[8:0],   TpTh[9]  = Threshold9[17:9], 
         DaqTh[10] = Threshold10[8:0],  TpTh[10] = Threshold10[17:9], 
         DaqTh[11] = Threshold11[8:0],  TpTh[11] = Threshold11[17:9], 
         DaqTh[12] = Threshold12[8:0],  TpTh[12] = Threshold12[17:9], 
         DaqTh[13] = Threshold13[8:0],  TpTh[13] = Threshold13[17:9], 
         DaqTh[14] = Threshold14[8:0],  TpTh[14] = Threshold14[17:9], 
         DaqTh[15] = Threshold15[8:0],  TpTh[15] = Threshold15[17:9], 
         DaqTh[16] = Threshold16[8:0],  TpTh[16] = Threshold16[17:9], 
         DaqTh[17] = Threshold17[8:0],  TpTh[17] = Threshold17[17:9], 
         DaqTh[18] = Threshold18[8:0],  TpTh[18] = Threshold18[17:9], 
         DaqTh[19] = Threshold19[8:0],  TpTh[19] = Threshold19[17:9], 
         DaqTh[20] = Threshold20[8:0],  TpTh[20] = Threshold20[17:9], 
         DaqTh[21] = Threshold21[8:0],  TpTh[21] = Threshold21[17:9], 
         DaqTh[22] = Threshold22[8:0],  TpTh[22] = Threshold22[17:9], 
         DaqTh[23] = Threshold23[8:0],  TpTh[23] = Threshold23[17:9], 
         DaqTh[24] = Threshold24[8:0],  TpTh[24] = Threshold24[17:9]; 

//********************** VERIFY TRIGGER RULES *******************//
// For the moment only Trigger rule i) and ii) of Trigger TDR; page 572:
// Rule 1: No more than 1 trigger per 3 BXs (=75 ns)
// Rule 2: No more than 2 triggers per 25 BXs (=625 ns)
// Input : Trigger    Output:  internal_legal_L1A,  RL
// Trigger must be 1-tick long (from MainCntr)
// RL set when an (L1A)Trigger is rejected, reset after an Trigger is accepted.
// RL_to_fifo is delayed so it is set long enough to be written into the pointer_fifo
// Terminology note: as the CalibTrig was introduced later (Nov 2004), most of
// the net names still refer to L1A.
   wire  Trigger = L1A | CalibTrig; 
   wire  L1A_in  = L1A_history[0];
   wire  L1A_out = L1A_history[23];  // 23 not 24. Simulate to believe!

always@(posedge clk)    begin 
   if(rst)    begin  
         L1A_d1 <= 0;   L1A_d2 <= 0;    RL <= 0; 
   // Dont reset  L1A_history and L1As_in_History : this keeps
   // rule-checking across reset. Also saves 23 FFs inferring SRL16.
         internal_legal_L1A <= 0; 
   end    // rst
   else   begin   // posedge clk
     L1A_d1 <= Trigger; L1A_d2 <= L1A_d1; // for L1A rule 1.
     L1A_history[23:0] <= {L1A_history[22:0], Trigger};  //  for L1A rule 2.
     if (L1A_in  && ~L1A_out) L1As_in_History <= L1As_in_History + 1;    
     if (L1A_out && ~L1A_in)  L1As_in_History <= L1As_in_History - 1; 
     if (Trigger) 
         if (L1A_d2 || (L1As_in_History >= 'd2)) RL <= 1;  // rule violation.
         else  internal_legal_L1A <= 1; 
     if (internal_legal_L1A) begin
         internal_legal_L1A <= 0; // internal_legal_L1A is 1-tick long
         RL <= 0;        // reset RL with next internal_legal_L1A
     end 
     RL_to_fifo <= RL;  // RL_to_fifo delayed (must wait next push of pointer_fifo)
   end     // posedge clk
end   // always


//********** WRITING into the DERANDOMIZER and into POINTER_FIFO **************//
// This process receives internal_legal_L1A and RL; it does not receive L1A.
// It manages the empty_event situation, not the trigger rules. 
// It controls the pointer fifo and input of Derand. buffer 
 
  always@(posedge clk)  begin 
      if(rst)   begin    // rst        
         stretch_cntr <= 'd0;   DrndWrAddr <= 'd0;    DrndWe <= 0; 
      end    // rst
    else    begin   // posedge clk
   /* here we stretch the Trigger pulse in order
      to write NS words, or 0 if an empty-event.  */
// legal_L1A pushes EmEv_to_fifo into pointer_fifo, together with RL and DrndWrAddr
  legal_L1A  <= internal_legal_L1A;  
  if(DrndWe)  DrndWrAddr <= DrndWrAddr + 1;  //always increase wr_addr while writing      

  if(internal_legal_L1A && ~BZ) begin // we enter the normal-event mode
      DrndWe <= 1; // the only case were we start writing into derand_buffers
      stretch_cntr <= 'd0;
      EmEv_to_fifo <= 0;
  end

  if(internal_legal_L1A && BZ) begin // the empty-event mode is 1-tick
      EmEv_to_fifo <= 1;    
      // Note that BZ can go on while transferring a normal event;
      // in this case we want to complete it:      
      if(DrndWe)  stretch_cntr <= stretch_cntr + 1;  // Continuation of a normal event
      //once written a # of samples = Block_size, disable derandomizer write:         
      if(stretch_cntr == NS-1)    DrndWe <= 0; 
  end

  if(~internal_legal_L1A) begin // this can be no-op or normal-event cont'd
      if(DrndWe) stretch_cntr <= stretch_cntr + 1;  // Continuation of a normal event
      //once written a # of samples = Block_size, disable derandomizer write:         
      if(stretch_cntr == NS-1)    DrndWe <= 0; 
  end
 
  end     // posedge clk
end   // always


//********** Generation of BUSY and Overflow_Warning flags *****************//
 assign WordCount =  DrndWrAddr - LatchedDrndPointer; 
 // Resolution of WordCount is ~ a block, not one word.
  
 always@(posedge clk)   begin
     if(rst)   WordCount_d <= 0;  // BZ and OW follow WordCount_d
     else begin  // "digital hysteresis"
        WordCount_d <= WordCount;  //register WordCount to increase Fmax.
        if( WordCount_d >= 9'd409)          BZ  <= 1;            
        if((WordCount_d <= 9'd307) && (BZ)) BZ  <= 0; 
        if( WordCount_d >= 9'd307)          OW  <= 1;            
        if((WordCount_d <= 9'd256) && (OW)) OW  <= 0;  
     end // else (not reset)
 end  // always 


//********  Data transfer from DERANDOMIZER to EventBuffer  *************//
// For every legal_L1A stored in the pointer_FIFO, we wait some time to 
// make sure in the DrndBuffer there is the Trigger Primitve that we 
// will evaluate for Zero-Suppression (worst case is if NPS=20). 
// If EventBuffer is empty, we register the DrndRdAddr (which is the output 
// of the pointer fifo) for each event. This registered address
// is used to generate all the derandomizer rd addresses. 
// Registering the address allows DaqFormatter to generate the
// Pointer FIFO Pop signal well in time for the next event.
// Then we move the data for each of the 48 channels (TP and DAQ) into
// the EventBuffer.
// When a complete event (fragment) is in the EbentBuffer, DaqCntr assert
// EvtRdy4Tx; DaqFormatter will acknowledge it sometime after (maybe it was
// in the middle of a previous transmission).                                                                       

 assign  DrndRdAddr = LatchedDrndPointer + SampleCntr; 
 
 always@(posedge clk)   begin   
   if(rst)     begin  // rst
         EvBufPushPre <= 0;           pendingL1A_20d <= 0;
         EventBufferPush <= 0;        EvtRdy4Tx <= 0;
         LoadingState <= IDLE;        LatchedDrndPointer <= 0;
   end   // rst
   else  begin  // posedge of clk          
         EventBufferPush      <= EvBufPushPre; 
         pendingL1Apipe[19:0] <= {pendingL1Apipe[18:0], ~pendingL1A_b};
         pendingL1A_20d       <= pendingL1Apipe[19];

 /* A FSM control the transfer of data from Drnd_Buffers to EventBuffer:
    - Make sure that it does not start at power-up
    - Begin operations if there is a "non-empty-event" and event_buffer is empty
    - Wait some 20 ticks, so that TP[NPS] is available for evaluation
    - Always go back to IDLE state   */
   case(LoadingState)
       IDLE: begin    
           ChanSel     <= 'd0;      EvBufPushPre  <= 0;
           SampleCntr  <= NPS;      EvtRdy4Tx     <= 0;
           TPwordCount <= 0;        DAQwordCount  <= 0;
           if(pendingL1A_20d && (~EmEv_from_pf) && EventBufferEmpty)                                
           begin
             LatchedDrndPointer <= DrndPointer; // latch it, in case it changes 
             if (NS < 'd11)       LoadingState <= N_CASE;  // NORMAL CASE
             else if (NS < 'd21)  LoadingState <= D_CASE; //DEBUGGING CASE
           end
         end // IDLE
                     
       N_CASE:  begin  // Normal CASE              
           LoadingState <= N_EVAL_TP; 
           TTh[8:0] <= TpTh[ChanSel+1]; 
         end
       N_EVAL_TP:  
           if (DaqMuxOut < TTh) LoadingState <= N_ONE_TP;
           else     begin       LoadingState <= N_NS_TP;    
                                SampleCntr   <= 5'd0;   end
         
       N_ONE_TP: begin
           SampleCntr   <= NPS;
           EvBufPushPre <= 1;  
           TPwordCount  <= TPwordCount + 8'b1;      
           LoadingState <= N_NEXT_TP;  
         end
       N_NS_TP: begin  // SampleCntr must be 0 when we enter here
           EvBufPushPre <= 1;  
           TPwordCount  <= TPwordCount + 8'b1;      
           if (SampleCntr == NS - 1) begin
                 LoadingState <= N_NEXT_TP;
                 SampleCntr  <= NPS;
           end
           else  SampleCntr <= SampleCntr + 1;  
         end
       N_NEXT_TP: begin
           EvBufPushPre <= 0;  
           SampleCntr   <= NPS;
           ChanSel <= ChanSel + 1;
           if (ChanSel == 'd23) LoadingState <= N_START_DAQ;
           else LoadingState <= N_CASE;  
         end
       N_START_DAQ: begin 
           LoadingState <= N_NOP;
           ChanSel <= ChanSel - 'd24; // prepare TP for evaluation
         end
       N_NOP: begin // needed to match delay of DaqMuxOut
           LoadingState <= N_EVAL_DAQ;
           DTh[8:0] <= DaqTh[ChanSel+1]; 
         end
       N_EVAL_DAQ: begin  
           ChanSel <= ChanSel + 'd24; // put it back, no matter what.           
           if (DaqMuxOut < DTh)      LoadingState <= N_NEXT_DAQ; 
           else     begin            LoadingState <= N_NS_DAQ;    
                                     SampleCntr   <= 5'd0;   end
         end
       N_NS_DAQ: begin  // SampleCntr must be 0 when we enter here
           EvBufPushPre <= 1;  
           DAQwordCount <= DAQwordCount + 9'b1;
           if (SampleCntr == NS - 1) begin
               LoadingState <= N_NEXT_DAQ;
               SampleCntr   <= NPS;
           end
           else  SampleCntr <= SampleCntr + 1;  
         end
       N_NEXT_DAQ: begin
           EvBufPushPre <= 0;  
           SampleCntr   <= NPS;
           ChanSel <= ChanSel + 1;
           if (ChanSel == 'd47)  begin
               LoadingState <= WAIT_ACK;
               EvtRdy4Tx <= 1;
               end
           else LoadingState <= N_START_DAQ;  
         end

       D_CASE:  begin  // Debugging CASE              
           LoadingState <= D_EVAL_TP; 
           TTh[8:0] <= TpTh[ChanSel+1]; 
         end
       D_EVAL_TP:  
           if (DaqMuxOut < TTh)   LoadingState <= D_NEXT_TP; 
           else                   LoadingState <= D_ONE_TP;    
         
       D_ONE_TP: begin
           SampleCntr   <= NPS;
           EvBufPushPre <= 1;  
           TPwordCount  <= TPwordCount + 8'b1;      
           LoadingState <= D_NEXT_TP;  
         end
       D_NEXT_TP: begin
           EvBufPushPre <= 0;  
           SampleCntr  <= NPS;
           ChanSel <= ChanSel + 1;
           if (ChanSel == 'd23) LoadingState <= D_START_DAQ;
           else LoadingState <= D_CASE;  
         end
       D_START_DAQ: begin 
           LoadingState <= D_NOP;
           ChanSel <= ChanSel - 'd24; // prepare TP for evaluation
         end
       D_NOP: begin // needed to match delay of DaqMuxOut
           LoadingState <= D_EVAL_DAQ;
           DTh[8:0] <= DaqTh[ChanSel+1]; 
         end
       D_EVAL_DAQ: begin  
           ChanSel <= ChanSel + 'd24; // put it back, no matter what!           
           if (DaqMuxOut < DTh)      LoadingState <= D_NEXT_DAQ; 
           else     begin            LoadingState <= D_NS_DAQ;    
                                     SampleCntr  <= 5'd0;   end
         end
       D_NS_DAQ: begin  // SampleCntr must be 0 when we enter here
           EvBufPushPre <= 1;  
           DAQwordCount <= DAQwordCount + 9'b1;
           if (SampleCntr == NS - 1) begin
               LoadingState <= D_NEXT_DAQ;
               SampleCntr   <= NPS;
           end
           else  SampleCntr <= SampleCntr + 1;  
         end
       D_NEXT_DAQ: begin
           EvBufPushPre <= 0;  
           SampleCntr   <= NPS;
           ChanSel <= ChanSel + 1;
           if (ChanSel == 'd47) begin
               LoadingState <= WAIT_ACK;
               EvtRdy4Tx <= 1;
            end
           else LoadingState <= D_START_DAQ;  
         end
       WAIT_ACK: 
          if (RdyAck)  LoadingState <= IDLE;

       default: LoadingState <= IDLE;
    endcase
    end   // posedge of clk
 end    // always  
endmodule   