// Title       : TriggerPath
// Description : 
// This file is the top level module for the Design of the Trigger Path.
// It contains the various modules for the trigger path processing 
// for all the 24 channels.  
//*******NOTE*******
// To correctly align the data going to the daqpath, the orignal data need
// to be delayed by the latency of the trigger path (about 5 cycles).
// ***verify the latency***
//-------------------------------------------------------------------------------------------------

`include "define.v"

module TriggerPath (
  CLK,        bypass,
  QIE1Data,  QIE2Data,  QIE3Data,  QIE4Data,  QIE5Data,  QIE6Data,
  QIE7Data,  QIE8Data,  QIE9Data,  QIE10Data, QIE11Data, QIE12Data,
  QIE13Data, QIE14Data, QIE15Data, QIE16Data, QIE17Data, QIE18Data,
  QIE19Data, QIE20Data, QIE21Data, QIE22Data, QIE23Data, QIE24Data,
  InLUT_Addr,      InLUT_Data,      InLUT_WE,    InLUT_Mode,
  InLut1DOut, InLut2DOut, InLut3DOut, InLut4DOut,
  InLut5DOut, InLut6DOut, InLut7DOut, InLut8DOut,
  InLut9DOut, InLut10DOut,InLut11DOut,InLut12DOut,
  InLut13DOut,InLut14DOut,InLut15DOut,InLut16DOut,
  InLut17DOut,InLut18DOut,InLut19DOut,InLut20DOut,
  InLut21DOut,InLut22DOut,InLut23DOut,InLut24DOut,
  OutLUT_Addr,         OutLUT_Data,         OutLUT_WE,   OutLUT_Mode,
  OutLUT_DOut1, OutLUT_DOut2, OutLUT_DOut3, OutLUT_DOut4,
  OutLUT_DOut5, OutLUT_DOut6, OutLUT_DOut7, OutLUT_DOut8,
  OutLUT_DOut9, OutLUT_DOut10,OutLUT_DOut11,OutLUT_DOut12,
  OutLUT_DOut13,OutLUT_DOut14,OutLUT_DOut15,OutLUT_DOut16,
  OutLUT_DOut17,OutLUT_DOut18,OutLUT_DOut19,OutLUT_DOut20,
  OutLUT_DOut21,OutLUT_DOut22,OutLUT_DOut23,OutLUT_DOut24,
  TP1_Data,  TP2_Data,  TP3_Data,  TP4_Data,  TP5_Data,  TP6_Data,
  TP7_Data,  TP8_Data,  TP9_Data,  TP10_Data, TP11_Data, TP12_Data,
  TP13_Data, TP14_Data, TP15_Data, TP16_Data, TP17_Data, TP18_Data,
  TP19_Data, TP20_Data, TP21_Data, TP22_Data, TP23_Data, TP24_Data,
  resetTP1, resetTP2, resetTP3, resetTP4,
  resetTP5, resetTP6, resetTP7, resetTP8           );

input CLK,  bypass;
input [ `QIEDATA_WIDTH - 1 : 0 ] 
        QIE1Data,  QIE2Data,  QIE3Data,  QIE4Data,  QIE5Data,  QIE6Data,
        QIE7Data,  QIE8Data,  QIE9Data,  QIE10Data, QIE11Data, QIE12Data,
        QIE13Data, QIE14Data, QIE15Data, QIE16Data, QIE17Data, QIE18Data,
        QIE19Data, QIE20Data, QIE21Data, QIE22Data, QIE23Data, QIE24Data ;
    
input [`INPUT_LUT_ADDR_WIDTH - 1 : 0 ]      InLUT_Addr;    
input [`INPUT_LUT_DATA_WIDTH - 1 : 0 ]      InLUT_Data;
input [24:1]      InLUT_WE;
input                                       InLUT_Mode;
output[`INPUT_LUT_DATA_WIDTH - 1 : 0 ]      
     InLut1DOut,  InLut2DOut,  InLut3DOut,  InLut4DOut,
     InLut5DOut,  InLut6DOut,  InLut7DOut,  InLut8DOut,
     InLut9DOut,  InLut10DOut, InLut11DOut, InLut12DOut,
     InLut13DOut, InLut14DOut, InLut15DOut, InLut16DOut,
     InLut17DOut, InLut18DOut, InLut19DOut, InLut20DOut,
     InLut21DOut, InLut22DOut, InLut23DOut, InLut24DOut; 

input [`COMPR_LUT_ADDR_WIDTH - 1 : 0 ]      OutLUT_Addr;
input [`COMPR_LUT_DATA_WIDTH - 1 : 0 ]      OutLUT_Data;
input [`NTT  : 1 ]    OutLUT_WE;
input                                       OutLUT_Mode;
output[`COMPR_LUT_DATA_WIDTH - 1 : 0 ]      
              OutLUT_DOut1, OutLUT_DOut2, OutLUT_DOut3, OutLUT_DOut4,
              OutLUT_DOut5, OutLUT_DOut6, OutLUT_DOut7, OutLUT_DOut8,
              OutLUT_DOut9, OutLUT_DOut10,OutLUT_DOut11,OutLUT_DOut12,
              OutLUT_DOut13,OutLUT_DOut14,OutLUT_DOut15,OutLUT_DOut16,
              OutLUT_DOut17,OutLUT_DOut18,OutLUT_DOut19,OutLUT_DOut20,
              OutLUT_DOut21,OutLUT_DOut22,OutLUT_DOut23,OutLUT_DOut24; 
input   resetTP1,resetTP2,resetTP3,resetTP4,
        resetTP5,resetTP6,resetTP7,resetTP8;   

output [`TRIGGER_PATH_DATA_WIDTH - 1 : 0 ]
       TP1_Data,  TP2_Data,  TP3_Data,  TP4_Data,  TP5_Data,  TP6_Data,
       TP7_Data,  TP8_Data,  TP9_Data,  TP10_Data, TP11_Data, TP12_Data,
      TP13_Data, TP14_Data, TP15_Data, TP16_Data, TP17_Data, TP18_Data,
      TP19_Data, TP20_Data, TP21_Data, TP22_Data, TP23_Data, TP24_Data;       

//trigger path for the 24 channels are instantiated below.
//each module implements the logic needed for generating a Trigger Primitives
TpgSingleChannel TPG1 (
    .QIEData(QIE1Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[1]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[1]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut1DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut1), 
    .TP(TP1_Data),
    .bypass(bypass),             .reset_TP(resetTP1)        );    
  
TpgSingleChannel TPG2 (
    .QIEData(QIE2Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[2]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[2]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut2DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut2), 
    .TP(TP2_Data),
    .bypass(bypass),             .reset_TP(resetTP1)         );      

TpgSingleChannel TPG3 (
    .QIEData(QIE3Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[3]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[3]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut3DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut3), 
    .TP(TP3_Data),
    .bypass(bypass),             .reset_TP(resetTP1)        );    

TpgSingleChannel TPG4 (
    .QIEData(QIE4Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[4]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[4]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut4DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut4), 
    .TP(TP4_Data),
    .bypass(bypass),             .reset_TP(resetTP2)     );  

TpgSingleChannel TPG5 (
    .QIEData(QIE5Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[5]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[5]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut5DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut5), 
    .TP(TP5_Data),
    .bypass(bypass),             .reset_TP(resetTP2)      );    
  
TpgSingleChannel TPG6 (
    .QIEData(QIE6Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[6]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[6]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut6DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut6), 
    .TP(TP6_Data),
    .bypass(bypass),             .reset_TP(resetTP2)       );      

TpgSingleChannel TPG7 (
    .QIEData(QIE7Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[7]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[7]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data), 
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut7DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut7),
    .TP(TP7_Data),
    .bypass(bypass),             .reset_TP(resetTP3)    );    

TpgSingleChannel TPG8 (
    .QIEData(QIE8Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[8]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[8]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut8DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut8), 
    .TP(TP8_Data),
    .bypass(bypass),             .reset_TP(resetTP3)    );  

TpgSingleChannel TPG9 (
    .QIEData(QIE9Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[9]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[9]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut9DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut9), 
    .TP(TP9_Data),
    .bypass(bypass),             .reset_TP(resetTP3)      );    
  
TpgSingleChannel TPG10 (
    .QIEData(QIE10Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[10]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[10]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut10DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut10), 
    .TP(TP10_Data),
    .bypass(bypass),             .reset_TP(resetTP4)      );      

TpgSingleChannel TPG11 (
    .QIEData(QIE11Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[11]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[11]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut11DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut11), 
    .TP(TP11_Data),
    .bypass(bypass),             .reset_TP(resetTP4)    );    

TpgSingleChannel TPG12 (
    .QIEData(QIE12Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[12]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[12]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data), 
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut12DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut12),
    .TP(TP12_Data),
    .bypass(bypass),             .reset_TP(resetTP4)      );  

TpgSingleChannel TPG13 (
    .QIEData(QIE13Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[13]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[13]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut13DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut13), 
    .TP(TP13_Data),
    .bypass(bypass),             .reset_TP(resetTP5)    );    
  
TpgSingleChannel TPG14 (
    .QIEData(QIE14Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[14]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[14]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut14DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut14), 
    .TP(TP14_Data),
    .bypass(bypass),             .reset_TP(resetTP5)      );  
      
TpgSingleChannel TPG15 (
    .QIEData(QIE15Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[15]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[15]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut15DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut15), 
    .TP(TP15_Data),
    .bypass(bypass),             .reset_TP(resetTP5)    );    

TpgSingleChannel TPG16 (
    .QIEData(QIE16Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[16]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[16]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut16DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut16), 
    .TP(TP16_Data),
    .bypass(bypass),             .reset_TP(resetTP6)    );

TpgSingleChannel TPG17 (
    .QIEData(QIE17Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[17]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[17]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut17DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut17), 
    .TP(TP17_Data),
    .bypass(bypass),             .reset_TP(resetTP6)      );    
  
TpgSingleChannel TPG18 (
    .QIEData(QIE18Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[18]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[18]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut18DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut18), 
    .TP(TP18_Data),
    .bypass(bypass),             .reset_TP(resetTP6)      );      

TpgSingleChannel TPG19 (
    .QIEData(QIE19Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[19]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[19]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut19DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut19), 
    .TP(TP19_Data),
    .bypass(bypass),             .reset_TP(resetTP7)      );    

TpgSingleChannel TPG20 (
    .QIEData(QIE20Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[20]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[20]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data), 
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut20DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut20),
    .TP(TP20_Data),
    .bypass(bypass),             .reset_TP(resetTP7)      );  

TpgSingleChannel TPG21 (
    .QIEData(QIE21Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[21]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[21]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data), 
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut21DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut21),
    .TP(TP21_Data),
    .bypass(bypass),             .reset_TP(resetTP7)      );    
  
TpgSingleChannel TPG22 (
    .QIEData(QIE22Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[22]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[22]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut22DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut22), 
    .TP(TP22_Data),
    .bypass(bypass),             .reset_TP(resetTP8)      );  

TpgSingleChannel TPG23 (
    .QIEData(QIE23Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[23]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[23]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut23DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut23), 
    .TP(TP23_Data),
    .bypass(bypass),             .reset_TP(resetTP8)      );

TpgSingleChannel TPG24 (
    .QIEData(QIE24Data), 
    .CLK(CLK), 
    .INPUT_LUT_INIT_ADDR(InLUT_Addr), 
    .InLutWE(InLUT_WE[24]), 
    .INPUT_LUT_INIT_DATA(InLUT_Data), 
    .COMPRESSION_LUT_INIT_ADDR(OutLUT_Addr), 
    .ComprLutWE(OutLUT_WE[24]), 
    .COMPRESSION_LUT_INIT_DATA(OutLUT_Data),
    .InLUT_Mode(InLUT_Mode),     .InLutDOut(InLut24DOut),
    .OutLUT_Mode(OutLUT_Mode),   .OutLUT_DOut(OutLUT_DOut24), 
    .TP(TP24_Data),
    .bypass(bypass),             .reset_TP(resetTP8)    );

endmodule